@@ -56,9 +56,11 @@ struct _PtyxisApplication
   PtyxisClient        *client;
   GHashTable          *exited;
   GVariant            *session;
+  GFileMonitor        *xdg_terminals_list_monitor;
   guint                has_restored_session : 1;
   guint                overlay_scrollbars : 1;
   guint                client_is_fallback : 1;
+  guint                maximize : 1;
 };
 
 static void ptyxis_application_about             (GSimpleAction *action,
@@ -70,6 +72,9 @@ static void ptyxis_application_edit_profile      (GSimpleAction *action,
 static void ptyxis_application_help_overlay      (GSimpleAction *action,
                                                   GVariant      *param,
                                                   gpointer       user_data);
+static void ptyxis_application_make_default      (GSimpleAction *action,
+                                                  GVariant      *param,
+                                                  gpointer       user_data);
 static void ptyxis_application_new_window_action (GSimpleAction *action,
                                                   GVariant      *param,
                                                   gpointer       user_data);
@@ -93,6 +98,7 @@ static GActionEntry action_entries[] = {
   { "focus-tab-by-uuid", ptyxis_application_focus_tab_by_uuid, "s" },
   { "new-window", ptyxis_application_new_window_action },
   { "new-tab", ptyxis_application_new_tab_action },
+  { "make-default", ptyxis_application_make_default },
 };
 
 G_DEFINE_FINAL_TYPE (PtyxisApplication, ptyxis_application, ADW_TYPE_APPLICATION)
@@ -120,6 +126,34 @@ ptyxis_application_new (const char        *application_id,
                        NULL);
 }
 
+static gboolean
+should_ignore_osc_title (PtyxisApplication *self,
+                         const char        *title)
+{
+  g_assert (PTYXIS_IS_APPLICATION (self));
+
+  if (ptyxis_settings_get_ignore_osc_title (self->settings))
+    return TRUE;
+
+  return !ptyxis_str_empty0 (title);
+}
+
+static void
+maybe_maximize (PtyxisApplication *self,
+                PtyxisWindow      *window)
+{
+  g_assert (PTYXIS_IS_APPLICATION (self));
+  g_assert (PTYXIS_IS_WINDOW (window));
+
+  if (self->maximize)
+    {
+      /*  unset maximize so it doesn't stick unless it's added again */
+      self->maximize = FALSE;
+
+      gtk_window_maximize (GTK_WINDOW (window));
+    }
+}
+
 static inline GFile *
 get_session_file (void)
 {
@@ -308,8 +342,11 @@ ptyxis_application_activate (GApplication *app)
       PtyxisTab *tab = ptyxis_window_get_active_tab (window);
 
       ptyxis_tab_set_title_prefix (tab, self->next_title_prefix);
+      ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, self->next_title_prefix));
       g_clear_pointer (&self->next_title_prefix, g_free);
 
+      maybe_maximize (self, window);
+
       gtk_window_present (GTK_WINDOW (window));
     }
 }
@@ -371,6 +408,7 @@ ptyxis_application_command_line (GApplication            *app,
   gboolean new_tab = FALSE;
   gboolean new_window = FALSE;
   gboolean did_restore = FALSE;
+  gboolean maximize = FALSE;
   int argc;
 
   g_assert (PTYXIS_IS_APPLICATION (self));
@@ -390,6 +428,9 @@ ptyxis_application_command_line (GApplication            *app,
   dict = g_application_command_line_get_options_dict (cmdline);
   argv = g_application_command_line_get_arguments (cmdline, &argc);
 
+  if (g_variant_dict_lookup (dict, "maximize", "b", &maximize))
+    self->maximize = !!maximize;
+
   if (!g_variant_dict_lookup (dict, "tab", "b", &new_tab))
     new_tab = FALSE;
 
@@ -468,6 +509,7 @@ ptyxis_application_command_line (GApplication            *app,
           tab = ptyxis_window_add_tab_for_command (window, NULL, (const char * const *)argv, cwd_uri);
 
           ptyxis_tab_set_title_prefix (tab, title);
+          ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
           gtk_window_present (GTK_WINDOW (window));
@@ -484,6 +526,7 @@ ptyxis_application_command_line (GApplication            *app,
           tab = ptyxis_window_add_tab_for_command (window, profile, (const char * const *)argv, cwd_uri);
 
           ptyxis_tab_set_title_prefix (tab, title);
+          ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
           gtk_window_present (GTK_WINDOW (window));
@@ -497,9 +540,12 @@ ptyxis_application_command_line (GApplication            *app,
           tab = ptyxis_window_add_tab_for_command (window, NULL, (const char * const *)argv, cwd_uri);
 
           ptyxis_tab_set_title_prefix (tab, title);
+          ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
           ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
           ptyxis_window_set_active_tab (window, tab);
 
+          maybe_maximize (self, window);
+
           gtk_window_present (GTK_WINDOW (window));
         }
       else
@@ -511,6 +557,7 @@ ptyxis_application_command_line (GApplication            *app,
           tab = ptyxis_window_get_active_tab (window);
 
           ptyxis_tab_set_title_prefix (tab, title);
+          ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
 
           gtk_application_add_window (GTK_APPLICATION (self), GTK_WINDOW (window));
           gtk_window_present (GTK_WINDOW (window));
@@ -531,6 +578,7 @@ ptyxis_application_command_line (GApplication            *app,
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
       ptyxis_tab_set_title_prefix (tab, title);
+      ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
 
@@ -551,6 +599,7 @@ ptyxis_application_command_line (GApplication            *app,
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
       ptyxis_tab_set_title_prefix (tab, title);
+      ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
 
@@ -571,9 +620,12 @@ ptyxis_application_command_line (GApplication            *app,
 
       ptyxis_tab_set_initial_working_directory_uri (tab, cwd_uri);
       ptyxis_tab_set_title_prefix (tab, title);
+      ptyxis_tab_set_ignore_osc_title (tab, should_ignore_osc_title (self, title));
       ptyxis_window_add_tab (window, tab);
       ptyxis_window_set_active_tab (window, tab);
 
+      maybe_maximize (self, window);
+
       gtk_window_present (GTK_WINDOW (window));
     }
   else
@@ -710,6 +762,24 @@ filter_session_container (gpointer item,
   return g_strcmp0 ("session", ptyxis_ipc_container_get_id (item)) != 0;
 }
 
+static void
+xdg_terminals_list_changed_cb (PtyxisApplication *self,
+                               GFile             *file,
+                               GFile             *other_file,
+                               GFileMonitorEvent  event,
+                               GFileMonitor      *monitor)
+{
+  GAction *action;
+
+  g_assert (PTYXIS_IS_APPLICATION (self));
+  g_assert (G_IS_FILE (file));
+  g_assert (G_IS_FILE_MONITOR (monitor));
+
+  if ((action = g_action_map_lookup_action (G_ACTION_MAP (self), "make-default")) &&
+      G_IS_SIMPLE_ACTION (action))
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !ptyxis_is_default ());
+}
+
 static gboolean
 ptyxis_application_should_sandbox_agent (PtyxisApplication *self)
 {
@@ -774,9 +844,10 @@ ptyxis_application_client_process_exited_cb (PtyxisApplication *self,
    * removed from the hash table when doing so (or when we get the real
    * value outside of a race condition).
    */
-  g_hash_table_insert (self->exited,
-                       g_strdup (process_object_path),
-                       GINT_TO_POINTER (exit_code));
+  if (self->exited != NULL)
+    g_hash_table_insert (self->exited,
+                         g_strdup (process_object_path),
+                         GINT_TO_POINTER (exit_code));
 }
 
 static void
@@ -789,12 +860,14 @@ ptyxis_application_startup (GApplication *application)
   g_autoptr(GFile) session_file = get_session_file ();
   g_autoptr(GBytes) session_bytes = NULL;
   g_autoptr(GError) error = NULL;
+  g_autoptr(GFile) xdg_terminals_list = NULL;
   AdwStyleManager *style_manager;
   gboolean sandbox_agent;
-  int timeout_msec;
 
   g_assert (PTYXIS_IS_APPLICATION (self));
 
+  xdg_terminals_list = g_file_new_build_filename (g_get_user_config_dir (), "xdg-terminals.list", NULL);
+
   g_application_set_default (application);
   g_application_set_resource_base_path (G_APPLICATION (self), "/org/gnome/Ptyxis");
 
@@ -802,6 +875,7 @@ ptyxis_application_startup (GApplication *application)
   self->profiles = g_list_store_new (PTYXIS_TYPE_PROFILE);
   self->settings = ptyxis_settings_new ();
   self->shortcuts = ptyxis_shortcuts_new (NULL);
+  self->xdg_terminals_list_monitor = g_file_monitor (xdg_terminals_list, 0, NULL, NULL);
 
   /* Load the session state so it's available if we need it */
   if ((session_bytes = g_file_load_bytes (session_file, NULL, NULL, NULL)))
@@ -814,14 +888,11 @@ ptyxis_application_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ptyxis_application_parent_class)->startup (application);
 
-  if ((sandbox_agent = ptyxis_application_should_sandbox_agent (self)))
-    timeout_msec = G_MAXINT;
-  else
-    timeout_msec = 2000;
+  sandbox_agent = ptyxis_application_should_sandbox_agent (self);
 
   /* Try to spawn ptyxis-agent on the host when possible */
   if (!(self->client = ptyxis_client_new (sandbox_agent, &error)) ||
-      !ptyxis_client_ping (self->client, timeout_msec, &error))
+      !ptyxis_client_ping (self->client, &error))
     {
       self->client_is_fallback = TRUE;
 
@@ -837,7 +908,7 @@ ptyxis_application_startup (GApplication *application)
       g_clear_error (&error);
 
       if (!(self->client = ptyxis_client_new (TRUE, &error)) ||
-          !ptyxis_client_ping (self->client, G_MAXINT, &error))
+          !ptyxis_client_ping (self->client, &error))
         g_error ("Failed to spawn ptyxis-agent in sandbox: %s", error->message);
     }
 
@@ -871,6 +942,20 @@ ptyxis_application_startup (GApplication *application)
                                          "app.help-overlay",
                                          (const char * const []) {"<ctrl>question", NULL});
 
+  if (self->xdg_terminals_list_monitor != NULL)
+    {
+      g_signal_connect_object (self->xdg_terminals_list_monitor,
+                               "changed",
+                               G_CALLBACK (xdg_terminals_list_changed_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
+      xdg_terminals_list_changed_cb (self,
+                                     xdg_terminals_list,
+                                     NULL,
+                                     G_FILE_MONITOR_EVENT_CHANGED,
+                                     self->xdg_terminals_list_monitor);
+    }
+
   /* Setup portal to get settings */
   self->portal = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                                 G_DBUS_PROXY_FLAGS_NONE,
@@ -929,6 +1014,7 @@ ptyxis_application_shutdown (GApplication *application)
 
   G_APPLICATION_CLASS (ptyxis_application_parent_class)->shutdown (application);
 
+  g_clear_object (&self->xdg_terminals_list_monitor);
   g_clear_object (&self->profile_menu);
   g_clear_object (&self->profiles);
   g_clear_object (&self->portal);
@@ -1034,9 +1120,10 @@ ptyxis_application_init (PtyxisApplication *self)
      */
     { "new-window", 0, 0, G_OPTION_ARG_NONE, NULL, N_("New terminal window") },
     { "tab", 0, 0, G_OPTION_ARG_NONE, NULL, N_("New terminal tab in active window") },
-    { "tab-with-profile", 0, 0, G_OPTION_ARG_STRING, NULL, N_("New terminal tab in active window using PROFILE"), N_("PROFILE") },
+    { "tab-with-profile", 0, 0, G_OPTION_ARG_STRING, NULL, N_("New terminal tab in active window using the profile UUID"), N_("PROFILE_UUID") },
 
     { "title", 'T', 0, G_OPTION_ARG_STRING, NULL, N_("Set title for new tab") },
+    { "maximize", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Maximize a newly created window") },
 
     { NULL }
   };
@@ -1840,11 +1927,14 @@ ptyxis_application_process_signaled_cb (PtyxisIpcProcess *process,
                                         int               term_sig,
                                         GTask            *task)
 {
+  PtyxisApplication *self = PTYXIS_APPLICATION_DEFAULT;
+
   g_assert (PTYXIS_IPC_IS_PROCESS (process));
   g_assert (G_IS_TASK (task));
 
-  g_hash_table_remove (PTYXIS_APPLICATION_DEFAULT->exited,
-                       g_dbus_proxy_get_object_path (G_DBUS_PROXY (process)));
+  if (self->exited != NULL)
+    g_hash_table_remove (self->exited,
+                         g_dbus_proxy_get_object_path (G_DBUS_PROXY (process)));
 
   wait_complete (task, 0, term_sig, NULL);
 }
@@ -2087,3 +2177,13 @@ ptyxis_application_get_user_data_dir (PtyxisApplication *self)
 
   return ptyxis_client_get_user_data_dir (self->client);
 }
+
+static void
+ptyxis_application_make_default (GSimpleAction *action,
+                                 GVariant      *param,
+                                 gpointer       user_data)
+{
+  g_assert (PTYXIS_IS_APPLICATION (user_data));
+
+  ptyxis_make_default ();
+}
