@@ -23,6 +23,8 @@
 
 #include <glib/gi18n.h>
 
+#include <cairo.h>
+
 #ifdef __linux__
 # include <libportal/portal.h>
 # include <libportal-gtk4/portal-gtk4.h>
@@ -79,6 +81,7 @@ struct _PtyxisTab
   PtyxisProcessLeaderKind  leader_kind : 3;
   guint                    has_foreground_process : 1;
   guint                    forced_exit : 1;
+  guint                    ignore_osc_title : 1;
   guint                    ignore_snapshot : 1;
 };
 
@@ -86,8 +89,12 @@ enum {
   PROP_0,
   PROP_COMMAND_LINE,
   PROP_ICON,
+  PROP_IGNORE_OSC_TITLE,
+  PROP_INDICATOR_ICON,
   PROP_PROCESS_LEADER_KIND,
   PROP_PROFILE,
+  PROP_PROGRESS,
+  PROP_PROGRESS_FRACTION,
   PROP_READ_ONLY,
   PROP_SUBTITLE,
   PROP_TITLE,
@@ -100,7 +107,6 @@ enum {
 
 enum {
   BELL,
-  COMMIT,
   N_SIGNALS
 };
 
@@ -232,6 +238,19 @@ ptyxis_tab_update_scrollback_lines (PtyxisTab *self)
   vte_terminal_set_scrollback_lines (VTE_TERMINAL (self->terminal), scrollback_lines);
 }
 
+static void
+ptyxis_tab_update_cell_height_scale (PtyxisTab *self)
+{
+  double cell_height_scale = 1.0;
+
+  g_assert (PTYXIS_IS_TAB (self));
+
+  if (ptyxis_profile_get_cell_height_scale (self->profile))
+    cell_height_scale = ptyxis_profile_get_cell_height_scale (self->profile);
+
+  vte_terminal_set_cell_height_scale (VTE_TERMINAL (self->terminal), cell_height_scale);
+}
+
 static gboolean
 ptyxis_tab_grab_focus (GtkWidget *widget)
 {
@@ -734,6 +753,20 @@ ptyxis_tab_update_padding_cb (PtyxisTab      *self,
     gtk_widget_add_css_class (GTK_WIDGET (self->terminal), "padded");
 }
 
+static void
+ptyxis_tab_update_word_char_exceptions (PtyxisTab      *self,
+                                        GParamSpec     *pspec,
+                                        PtyxisSettings *settings)
+{
+  g_autofree char *word_char_exceptions = NULL;
+
+  g_assert (PTYXIS_IS_TAB (self));
+  g_assert (PTYXIS_IS_SETTINGS (settings));
+
+  word_char_exceptions = ptyxis_settings_dup_word_char_exceptions (settings);
+  vte_terminal_set_word_char_exceptions (VTE_TERMINAL (self->terminal), word_char_exceptions);
+}
+
 static void
 ptyxis_tab_constructed (GObject *object)
 {
@@ -761,6 +794,9 @@ ptyxis_tab_constructed (GObject *object)
   g_object_bind_property (settings, "text-blink-mode",
                           self->terminal, "text-blink-mode",
                           G_BINDING_SYNC_CREATE);
+  g_object_bind_property (settings, "ignore-osc-title",
+                          self, "ignore-osc-title",
+                          G_BINDING_SYNC_CREATE);
 
   g_signal_connect_object (settings,
                            "notify::disable-padding",
@@ -793,6 +829,20 @@ ptyxis_tab_constructed (GObject *object)
                            G_CONNECT_SWAPPED);
   ptyxis_tab_update_scrollback_lines (self);
 
+  g_signal_connect_object (G_OBJECT (self->profile),
+                           "notify::cell-height-scale",
+                           G_CALLBACK (ptyxis_tab_update_cell_height_scale),
+                           self,
+                           G_CONNECT_SWAPPED);
+  ptyxis_tab_update_cell_height_scale (self);
+
+  g_signal_connect_object (settings,
+                           "notify::word-char-exceptions",
+                           G_CALLBACK (ptyxis_tab_update_word_char_exceptions),
+                           self,
+                           G_CONNECT_SWAPPED);
+  ptyxis_tab_update_word_char_exceptions (self, NULL, settings);
+
   self->monitor = ptyxis_tab_monitor_new (self);
 }
 
@@ -896,6 +946,16 @@ ptyxis_tab_invalidate_icon (PtyxisTab *self)
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ICON]);
 }
 
+static void
+ptyxis_tab_invalidate_progress (PtyxisTab *self)
+{
+  g_assert (PTYXIS_IS_TAB (self));
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PROGRESS]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PROGRESS_FRACTION]);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_INDICATOR_ICON]);
+}
+
 static gboolean
 ptyxis_tab_match_clicked_cb (PtyxisTab       *self,
                              double           x,
@@ -931,18 +991,6 @@ ptyxis_tab_root (GtkWidget *widget)
   GTK_WIDGET_CLASS (ptyxis_tab_parent_class)->root (widget);
 }
 
-static void
-ptyxis_tab_commit_cb (PtyxisTab      *self,
-                      const char     *str,
-                      guint           length,
-                      PtyxisTerminal *terminal)
-{
-  g_assert (PTYXIS_IS_TAB (self));
-  g_assert (PTYXIS_IS_TERMINAL (terminal));
-
-  g_signal_emit (self, signals[COMMIT], 0, str);
-}
-
 static void
 ptyxis_tab_dispose (GObject *object)
 {
@@ -1005,10 +1053,26 @@ ptyxis_tab_get_property (GObject    *object,
       g_value_take_object (value, ptyxis_tab_dup_icon (self));
       break;
 
+    case PROP_IGNORE_OSC_TITLE:
+      g_value_set_boolean (value, ptyxis_tab_get_ignore_osc_title (self));
+      break;
+
+    case PROP_INDICATOR_ICON:
+      g_value_take_object (value, ptyxis_tab_dup_indicator_icon (self));
+      break;
+
     case PROP_PROCESS_LEADER_KIND:
       g_value_set_enum (value, self->leader_kind);
       break;
 
+    case PROP_PROGRESS:
+      g_value_set_enum (value, ptyxis_tab_get_progress (self));
+      break;
+
+    case PROP_PROGRESS_FRACTION:
+      g_value_set_double (value, ptyxis_tab_get_progress_fraction (self));
+      break;
+
     case PROP_PROFILE:
       g_value_set_object (value, ptyxis_tab_get_profile (self));
       break;
@@ -1056,6 +1120,10 @@ ptyxis_tab_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_IGNORE_OSC_TITLE:
+      ptyxis_tab_set_ignore_osc_title (self, g_value_get_boolean (value));
+      break;
+
     case PROP_PROFILE:
       self->profile = g_value_dup_object (value);
       break;
@@ -1107,6 +1175,19 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
                          (G_PARAM_READABLE |
                           G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_IGNORE_OSC_TITLE] =
+    g_param_spec_boolean ("ignore-osc-title", NULL, NULL,
+                         FALSE,
+                         (G_PARAM_READWRITE |
+                          G_PARAM_EXPLICIT_NOTIFY |
+                          G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_INDICATOR_ICON] =
+    g_param_spec_object ("indicator-icon", NULL, NULL,
+                         G_TYPE_ICON,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
   properties[PROP_PROCESS_LEADER_KIND] =
     g_param_spec_enum ("process-leader-kind", NULL, NULL,
                        PTYXIS_TYPE_PROCESS_LEADER_KIND,
@@ -1121,6 +1202,19 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
                           G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS));
 
+  properties[PROP_PROGRESS] =
+    g_param_spec_enum ("progress", NULL, NULL,
+                       PTYXIS_TYPE_TAB_PROGRESS,
+                       PTYXIS_TAB_PROGRESS_INDETERMINATE,
+                       (G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS));
+
+  properties[PROP_PROGRESS_FRACTION] =
+    g_param_spec_double ("progress-fraction", NULL, NULL,
+                         0, 1, 0,
+                         (G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
+
   properties[PROP_READ_ONLY] =
     g_param_spec_boolean ("read-only", NULL, NULL,
                           FALSE,
@@ -1178,17 +1272,6 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
                                 NULL,
                                 G_TYPE_NONE, 0);
 
-  signals[COMMIT] =
-    g_signal_new_class_handler ("commit",
-                                G_TYPE_FROM_CLASS (klass),
-                                G_SIGNAL_RUN_LAST,
-                                NULL,
-                                NULL, NULL,
-                                NULL,
-                                G_TYPE_NONE,
-                                1,
-                                G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
-
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Ptyxis/ptyxis-tab.ui");
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "ptyxistab");
@@ -1205,8 +1288,8 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_notify_palette_cb);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_bell_cb);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_invalidate_icon);
+  gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_invalidate_progress);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_match_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_commit_cb);
 
   gtk_widget_class_install_action (widget_class, "tab.respawn", NULL, ptyxis_tab_respawn_action);
   gtk_widget_class_install_action (widget_class, "tab.inspect", NULL, ptyxis_tab_inspect_action);
@@ -1288,8 +1371,8 @@ ptyxis_tab_get_title_prefix (PtyxisTab *self)
 }
 
 void
-ptyxis_tab_set_title_prefix (PtyxisTab *self,
-                              const char *title_prefix)
+ptyxis_tab_set_title_prefix (PtyxisTab  *self,
+                             const char *title_prefix)
 {
   g_return_if_fail (PTYXIS_IS_TAB (self));
 
@@ -1306,24 +1389,29 @@ ptyxis_tab_set_title_prefix (PtyxisTab *self,
 char *
 ptyxis_tab_dup_title (PtyxisTab *self)
 {
-  const char *window_title;
   GString *gstr;
 
   g_return_val_if_fail (PTYXIS_IS_TAB (self), NULL);
 
   gstr = g_string_new (self->title_prefix);
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  window_title = vte_terminal_get_window_title (VTE_TERMINAL (self->terminal));
-  G_GNUC_END_IGNORE_DEPRECATIONS
+  if (!self->ignore_osc_title)
+    {
+      const char *window_title;
 
-  if (window_title && window_title[0])
-    g_string_append (gstr, window_title);
-  else if (self->command != NULL && self->command[0] != NULL)
-    g_string_append (gstr, self->command[0]);
-  else if (self->initial_title != NULL)
-    g_string_append (gstr, self->initial_title);
-  else
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+        window_title = vte_terminal_get_window_title (VTE_TERMINAL (self->terminal));
+      G_GNUC_END_IGNORE_DEPRECATIONS
+
+      if (window_title && window_title[0])
+        g_string_append (gstr, window_title);
+      else if (self->command != NULL && self->command[0] != NULL)
+        g_string_append (gstr, self->command[0]);
+      else if (self->initial_title != NULL)
+        g_string_append (gstr, self->initial_title);
+    }
+
+  if (gstr->len == 0)
     g_string_append (gstr, _("Terminal"));
 
   if (self->state == PTYXIS_TAB_STATE_EXITED)
@@ -2002,6 +2090,193 @@ ptyxis_tab_query_working_directory_from_agent (PtyxisTab *self)
   return NULL;
 }
 
+PtyxisTabProgress
+ptyxis_tab_get_progress (PtyxisTab *self)
+{
+  gint64 state;
+
+  g_return_val_if_fail (PTYXIS_IS_TAB (self), 0);
+
+  if (vte_terminal_get_termprop_int_by_id (VTE_TERMINAL (self->terminal),
+                                           VTE_PROPERTY_ID_PROGRESS_HINT,
+                                           &state))
+    {
+      switch (state)
+        {
+        case VTE_PROGRESS_HINT_ACTIVE:
+          return PTYXIS_TAB_PROGRESS_ACTIVE;
+
+        case VTE_PROGRESS_HINT_ERROR:
+          return PTYXIS_TAB_PROGRESS_ERROR;
+
+        case VTE_PROGRESS_HINT_PAUSED:
+        case VTE_PROGRESS_HINT_INDETERMINATE:
+        default:
+          return PTYXIS_TAB_PROGRESS_INDETERMINATE;
+        }
+    }
+
+  return PTYXIS_TAB_PROGRESS_INDETERMINATE;
+}
+
+double
+ptyxis_tab_get_progress_fraction (PtyxisTab *self)
+{
+  guint64 value;
+
+  g_return_val_if_fail (PTYXIS_IS_TAB (self), .0);
+
+  if (ptyxis_tab_get_progress (self) != PTYXIS_TAB_PROGRESS_ACTIVE ||
+      !vte_terminal_get_termprop_uint_by_id (VTE_TERMINAL (self->terminal),
+                                             VTE_PROPERTY_ID_PROGRESS_VALUE,
+                                             &value))
+    return .0;
+
+  return MIN (value, 100) / 100.0;
+}
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+static void
+draw_progress (cairo_t         *cr,
+               GtkStyleContext *style_context,
+               int              width,
+               int              height,
+               double           progress)
+{
+  GdkRGBA rgba;
+  double alpha;
+
+  g_assert (cr != NULL);
+  g_assert (style_context != NULL);
+
+  progress = CLAMP (progress, 0, 1);
+
+  gtk_style_context_get_color (style_context, &rgba);
+
+  alpha = rgba.alpha;
+  rgba.alpha *= .15;
+  gdk_cairo_set_source_rgba (cr, &rgba);
+
+  cairo_arc (cr,
+             width / 2,
+             height / 2,
+             width / 2,
+             0.0,
+             2 * M_PI);
+  cairo_fill (cr);
+
+  if (progress > 0.0)
+    {
+      rgba.alpha = alpha;
+      gdk_cairo_set_source_rgba (cr, &rgba);
+
+      cairo_arc (cr,
+                 width / 2,
+                 height / 2,
+                 width / 2,
+                 (-.5 * M_PI),
+                 (2 * progress * M_PI) - (.5 * M_PI));
+
+      if (progress != 1.0)
+        {
+          cairo_line_to (cr, width / 2, height / 2);
+          cairo_line_to (cr, width / 2, 0);
+        }
+
+      cairo_fill (cr);
+    }
+}
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+/**
+ * ptyxis_tab_dup_indicator_icon:
+ * @self: a #PtyxisTab
+ *
+ * Gets the progress indicator icon.
+ *
+ * Due to libadwaita not providing a way to do progress natively (as of 1.6)
+ * this uses indicator icon to generate a progress icon using a drawing.
+ *
+ * Returns: (transfer full) (nullable): a #GIcon or %NULL
+ */
+GIcon *
+ptyxis_tab_dup_indicator_icon (PtyxisTab *self)
+{
+  PtyxisTabProgress progress;
+
+  g_return_val_if_fail (PTYXIS_IS_TAB (self), NULL);
+
+  progress = ptyxis_tab_get_progress (self);
+
+  if (progress == PTYXIS_TAB_PROGRESS_ERROR)
+    return g_themed_icon_new ("dialog-error-symbolic");
+
+  if (progress == PTYXIS_TAB_PROGRESS_INDETERMINATE)
+    return NULL;
+
+  if (progress == PTYXIS_TAB_PROGRESS_ACTIVE)
+    {
+      g_autoptr(GdkTexture) texture = NULL;
+      g_autoptr(GBytes) bytes = NULL;
+      cairo_surface_t *surface;
+      cairo_t *cr;
+      double fraction;
+      int stride;
+      int scale;
+      int width;
+      int height;
+
+      fraction = ptyxis_tab_get_progress_fraction (self);
+      scale = gtk_widget_get_scale_factor (GTK_WIDGET (self));
+      width = 16 * scale;
+      height = 16 * scale;
+
+      surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+      stride = cairo_image_surface_get_stride (surface);
+      cr = cairo_create (surface);
+
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS {
+        GtkStyleContext *style_context = gtk_widget_get_style_context (GTK_WIDGET (self));
+        draw_progress (cr, style_context, width, height, fraction);
+      } G_GNUC_END_IGNORE_DEPRECATIONS
+
+      cairo_destroy (cr);
+
+      bytes = g_bytes_new (cairo_image_surface_get_data (surface), height * stride);
+      texture = gdk_memory_texture_new (width, height, GDK_MEMORY_DEFAULT, bytes, stride);
+
+      cairo_surface_destroy (surface);
+
+      return G_ICON (g_steal_pointer (&texture));
+    }
+
+  return NULL;
+}
+
+gboolean
+ptyxis_tab_get_ignore_osc_title (PtyxisTab *self)
+{
+  g_return_val_if_fail (PTYXIS_IS_TAB (self), FALSE);
+
+  return self->ignore_osc_title;
+}
+
+void
+ptyxis_tab_set_ignore_osc_title (PtyxisTab *self,
+                                 gboolean   ignore_osc_title)
+{
+  g_return_if_fail (PTYXIS_IS_TAB (self));
+
+  ignore_osc_title = !!ignore_osc_title;
+
+  if (ignore_osc_title != self->ignore_osc_title)
+    {
+      self->ignore_osc_title = ignore_osc_title;
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_IGNORE_OSC_TITLE]);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_TITLE]);
+    }
+}
+
 void
 _ptyxis_tab_ignore_snapshot (PtyxisTab *self)
 {
