@@ -153,6 +153,7 @@ ptyxis_window_close_page_cb (PtyxisWindow *self,
                              AdwTabView   *tab_view)
 {
   g_autoptr(GPtrArray) tabs = NULL;
+  PtyxisSettings *settings;
   PtyxisTab *tab;
 
   g_assert (PTYXIS_IS_WINDOW (self));
@@ -162,12 +163,14 @@ ptyxis_window_close_page_cb (PtyxisWindow *self,
   ptyxis_window_save_size (self);
 
   tab = PTYXIS_TAB (adw_tab_page_get_child (tab_page));
+  settings = ptyxis_application_get_settings (PTYXIS_APPLICATION_DEFAULT);
 
   /* If we are disposed, allow closing immediately */
   if (self->disposed)
     return GDK_EVENT_PROPAGATE;
 
-  if (!ptyxis_tab_is_running (tab, NULL))
+  if (!ptyxis_tab_is_running (tab, NULL) ||
+      !ptyxis_settings_get_prompt_on_close (settings))
     {
       ptyxis_parking_lot_push (self->parking_lot, tab);
       gtk_window_set_default_size (GTK_WINDOW (self), -1, -1);
@@ -391,17 +394,10 @@ ptyxis_window_page_attached_cb (PtyxisWindow *self,
                                 int           position,
                                 AdwTabView   *tab_view)
 {
-  GtkWidget *child;
-
   g_assert (PTYXIS_IS_WINDOW (self));
   g_assert (ADW_IS_TAB_PAGE (page));
   g_assert (ADW_IS_TAB_VIEW (tab_view));
 
-  child = adw_tab_page_get_child (page);
-
-  g_object_bind_property (child, "title", page, "title", G_BINDING_SYNC_CREATE);
-  g_object_bind_property (child, "icon", page, "icon", G_BINDING_SYNC_CREATE);
-
   update_visible_and_maybe_close (self);
 }
 
@@ -786,18 +782,6 @@ ptyxis_window_active_tab_bell_cb (PtyxisWindow *self,
   ptyxis_window_visual_bell (self);
 }
 
-static void
-ptyxis_window_active_tab_commit_cb (PtyxisWindow *self,
-                                    const char   *str,
-                                    PtyxisTab    *tab)
-{
-  g_assert (PTYXIS_IS_WINDOW (self));
-  g_assert (PTYXIS_IS_TAB (tab));
-
-  if (self->is_fullscreen)
-    ptyxis_fullscreen_box_unreveal (self->fullscreen_box);
-}
-
 static void
 ptyxis_window_close_action (GtkWidget  *widget,
                             const char *action_name,
@@ -1223,6 +1207,18 @@ ptyxis_window_primary_menu_action (GtkWidget  *widget,
   gtk_menu_button_popup (self->primary_menu_button);
 }
 
+static void
+ptyxis_window_tab_menu_action (GtkWidget  *widget,
+                               const char *action_name,
+                               GVariant   *param)
+{
+  PtyxisWindow *self = (PtyxisWindow *)widget;
+
+  g_assert (PTYXIS_IS_WINDOW (self));
+
+  gtk_menu_button_popup (self->new_terminal_menu_button);
+}
+
 static void
 ptyxis_window_close_request_cb (GObject      *object,
                                 GAsyncResult *result,
@@ -1261,6 +1257,7 @@ ptyxis_window_close_request (GtkWindow *window)
 {
   PtyxisWindow *self = (PtyxisWindow *)window;
   g_autoptr(GPtrArray) tabs = NULL;
+  PtyxisSettings *settings;
   guint n_pages;
 
   g_assert (PTYXIS_IS_WINDOW (self));
@@ -1270,6 +1267,11 @@ ptyxis_window_close_request (GtkWindow *window)
   if (!self->single_terminal_mode && is_last_window (self))
     ptyxis_application_save_session (PTYXIS_APPLICATION_DEFAULT);
 
+  /* Short-circuit if user dismissed guard rails */
+  settings = ptyxis_application_get_settings (PTYXIS_APPLICATION_DEFAULT);
+  if (!ptyxis_settings_get_prompt_on_close (settings))
+    return GDK_EVENT_PROPAGATE;
+
   tabs = g_ptr_array_new_with_free_func (g_object_unref);
   n_pages = adw_tab_view_get_n_pages (self->tab_view);
 
@@ -1288,10 +1290,10 @@ ptyxis_window_close_request (GtkWindow *window)
     return GDK_EVENT_PROPAGATE;
 
   _ptyxis_close_dialog_run_async (GTK_WINDOW (self),
-                                   tabs,
-                                   NULL,
-                                   ptyxis_window_close_request_cb,
-                                   g_object_ref (self));
+                                  tabs,
+                                  NULL,
+                                  ptyxis_window_close_request_cb,
+                                  g_object_ref (self));
 
   return GDK_EVENT_STOP;
 }
@@ -1465,30 +1467,6 @@ ptyxis_window_update_menu_visibility (PtyxisWindow *self)
   gtk_widget_set_visible (GTK_WIDGET (self->new_terminal_menu_button), visible);
 }
 
-static void
-notify_decoration_layout_cb (PtyxisWindow *self,
-                             GParamSpec   *pspec,
-                             GtkSettings  *gtk_settings)
-{
-  g_autofree char *layout = NULL;
-  gboolean inverted = FALSE;
-  const char *colon;
-  const char *close_;
-
-  g_assert (PTYXIS_IS_WINDOW (self));
-  g_assert (GTK_IS_SETTINGS (gtk_settings));
-
-  g_object_get (gtk_settings,
-                "gtk-decoration-layout", &layout,
-                NULL);
-
-  if ((colon = strchr (layout, ':')) && (close_ = strstr (layout, "close")))
-    inverted = close_ < colon;
-
-  if (self->tab_bar)
-    adw_tab_bar_set_inverted (self->tab_bar, inverted);
-}
-
 static void
 ptyxis_window_constructed (GObject *object)
 {
@@ -1499,7 +1477,6 @@ ptyxis_window_constructed (GObject *object)
   g_autoptr(GMenuModel) profile_menu = NULL;
   g_autoptr(GMenuModel) container_menu = NULL;
   g_autoptr(GMenu) menu = NULL;
-  GtkSettings *gtk_settings;
 
   G_OBJECT_CLASS (ptyxis_window_parent_class)->constructed (object);
 
@@ -1539,14 +1516,6 @@ ptyxis_window_constructed (GObject *object)
                            self,
                            G_CONNECT_SWAPPED);
   ptyxis_window_update_menu_visibility (self);
-
-  gtk_settings = gtk_settings_get_default ();
-  g_signal_connect_object (gtk_settings,
-                           "notify::gtk-decoration-layout",
-                           G_CALLBACK (notify_decoration_layout_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
-  notify_decoration_layout_cb (self, NULL, gtk_settings);
 }
 
 static void
@@ -1757,6 +1726,7 @@ ptyxis_window_class_init (PtyxisWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_window_tab_overview_create_tab_cb);
 
   gtk_widget_class_install_action (widget_class, "win.primary-menu", NULL, ptyxis_window_primary_menu_action);
+  gtk_widget_class_install_action (widget_class, "win.tab-menu", NULL, ptyxis_window_tab_menu_action);
   gtk_widget_class_install_action (widget_class, "win.new-tab", "(ss)", ptyxis_window_new_tab_action);
   gtk_widget_class_install_action (widget_class, "win.new-window", "(ss)", ptyxis_window_new_window_action);
   gtk_widget_class_install_action (widget_class, "win.new-terminal", "(ss)", ptyxis_window_new_terminal_action);
@@ -1821,11 +1791,6 @@ ptyxis_window_init (PtyxisWindow *self)
                                  G_CALLBACK (ptyxis_window_active_tab_bell_cb),
                                  self,
                                  G_CONNECT_SWAPPED);
-  g_signal_group_connect_object (self->active_tab_signals,
-                                 "commit",
-                                 G_CALLBACK (ptyxis_window_active_tab_commit_cb),
-                                 self,
-                                 G_CONNECT_SWAPPED);
   g_signal_group_connect_object (self->active_tab_signals,
                                  "notify::process-leader-kind",
                                  G_CALLBACK (ptyxis_window_notify_process_leader_kind_cb),
@@ -1968,14 +1933,32 @@ ptyxis_window_new_for_profile_and_command (PtyxisProfile      *profile,
   return self;
 }
 
+static void
+ptyxis_window_setup_page (PtyxisWindow *self,
+                          PtyxisTab    *tab,
+                          AdwTabPage   *page)
+{
+  g_assert (PTYXIS_IS_WINDOW (self));
+  g_assert (PTYXIS_IS_TAB (tab));
+  g_assert (ADW_IS_TAB_PAGE (page));
+
+  g_object_bind_property (tab, "title", page, "title", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (tab, "icon", page, "icon", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (tab, "indicator-icon", page, "indicator-icon", G_BINDING_SYNC_CREATE);
+}
+
 void
 ptyxis_window_append_tab (PtyxisWindow *self,
                           PtyxisTab    *tab)
 {
+  AdwTabPage *page;
+
   g_return_if_fail (PTYXIS_IS_WINDOW (self));
   g_return_if_fail (PTYXIS_IS_TAB (tab));
 
-  adw_tab_view_append (self->tab_view, GTK_WIDGET (tab));
+  page = adw_tab_view_append (self->tab_view, GTK_WIDGET (tab));
+
+  ptyxis_window_setup_page (self, tab, page);
 
   gtk_widget_grab_focus (GTK_WIDGET (tab));
 }
@@ -2079,7 +2062,9 @@ ptyxis_window_add_tab (PtyxisWindow *self,
         }
     }
 
-  adw_tab_view_insert (self->tab_view, GTK_WIDGET (tab), position);
+  page = adw_tab_view_insert (self->tab_view, GTK_WIDGET (tab), position);
+
+  ptyxis_window_setup_page (self, tab, page);
 
   /* Resize if we are going from 1->2 tabs */
   if (adw_tab_view_get_n_pages (self->tab_view) == 2)
